@@ -3,7 +3,7 @@ package WiringPi::API;
 use strict;
 use warnings;
 
-our $VERSION = '2.36.2';
+our $VERSION = '2.36.3';
 
 require XSLoader;
 XSLoader::load('WiringPi::API', $VERSION);
@@ -25,7 +25,7 @@ my @wpi_c_functions = qw(
     sr595Setup          bmp180Setup         bmp180Pressure
     bmp180Temp          analogRead          analogWrite
     physPinToWpi        wiringPiVersion     ads1115Setup
-    pseudoPinsSetup
+    pseudoPinsSetup     wiringPiSPISetup    spiDataRW
 );
 
 my @wpi_perl_functions = qw(
@@ -38,7 +38,7 @@ my @wpi_perl_functions = qw(
     set_interrupt   soft_pwm_create soft_pwm_write      soft_pwm_stop
     shift_reg_setup bmp180_setup    bmp180_pressure     bmp180_temp
     analog_read     analog_write    pin_mode            phys_to_wpi
-    ads1115_setup
+    ads1115_setup   spi_setup       spi_data
 );
 
 our @EXPORT_OK;
@@ -256,6 +256,43 @@ sub shift_reg_setup {
     sr595Setup($pin_base, $num_pins, $data_pin, $clock_pin, $latch_pin);
 }
 
+# SPI functions
+
+sub spi_setup {
+    shift if @_ == 3;
+    my ($channel, $speed) = @_;
+
+    if ($channel != 0 && $channel != 1){
+        die "spi_setup() channel param must be 0 or 1\n";
+    }
+
+    $speed = 1000000 if ! defined $speed;
+
+    return wiringPiSPISetup($channel, $speed);
+}
+sub spi_data {
+    shift if @_ == 4;
+    my ($chan, $data, $len) = @_;
+
+    if ($chan != 0 && $chan != 1){
+        die "spi_data() channel param must be 0 or 1\n";
+    }
+    if (ref $data ne 'ARRAY'){
+        die "spi_data() data param must be an array reference\n";
+    }
+    if (@$data != $len){
+        die "spi_data() array reference must have \$len param count\n";
+    }
+
+    my $buf;
+
+    for (@$data){
+        $buf += $_;
+    }
+
+    return spiDataRW($chan, $buf, $len);
+}
+
 # bmp180 pressure sensor functions
 
 sub bmp180_setup {
@@ -308,8 +345,8 @@ __END__
 
 =head1 NAME
 
-WiringPi::API - Direct access to Raspberry Pi's wiringPi API, with optional
-Perl OO access
+WiringPi::API - API for wiringPi, providing access to the Raspberry Pi's board,
+GPIO and connected peripherals
 
 =head1 SYNOPSIS
 
@@ -340,9 +377,9 @@ you must initialize the software by calling one of the C<setup*()> routines.
 
 =head1 DESCRIPTION
 
-This is an XS-based module, and requires L<wiringPi|http://wiringpi.com> version 2.36+
-to be installed. The C<wiringPiDev> shared library is also required (for the LCD
-functionality), but it's installed by default with C<wiringPi>.
+This is an XS-based module, and requires L<wiringPi|http://wiringpi.com> version
+2.36+ to be installed. The C<wiringPiDev> shared library is also required (for
+the LCD functionality), but it's installed by default with C<wiringPi>.
 
 This module allows you to import the wiringPi's functions directly as-is, use
 it as a Perl base class, export the Perl wrapped functions, or use it in a
@@ -358,8 +395,11 @@ from the C code itself.
 Exported with the C<:wiringPi> tag.
 
 These XS functions map directly to the wiringPi functions with their original
-names. Note that C<setInterrupt> is not a direct wrapper, it's a custom C
-wrapper for C<wiringPiISR()> in order to make it functional here.
+names. 
+
+Note that C<setInterrupt> is a custom C wrapper for C<wiringPiISR()>, as is
+C<spiDataRW> for C<wiringPiSPIDataRW(). This is required in order to be able
+to make them usable in Perl land.
 
     wiringPiSetup       wiringPiSetupSys    wiringPiSetupGpio
     wiringPiSetupPhys   pinMode             pullUpDnControl
@@ -373,6 +413,8 @@ wrapper for C<wiringPiISR()> in order to make it functional here.
     softPwmCreate       softPwmWrite        softPwmStop
     sr595Setup          bmp180Setup         bmp180Pressure
     bmp180Temp          analogRead          analogWrite
+    ads1115Setup        pseudoPinsSetup     wiringPiSPISetup
+    wiringPiSPIDataRW
 
 Exported with the C<:perl> tag.
 
@@ -388,7 +430,8 @@ versions, but are still 100% compatible.
     lcd_position    lcd_char_def    lcd_put_char        lcd_puts
     set_interrupt   soft_pwm_create soft_pwm_write      soft_pwm_stop
     shift_reg_setup pin_mode        analog_read         analog_write
-    bmp180_setup    bmp180_pressure bmp180_temp
+    bmp180_setup    bmp180_pressure bmp180_temp         phys_to_wpi
+    ads1115_setup   spi_setup       spi_data
 
 =head1 EXPORT_TAGS
 
@@ -404,7 +447,7 @@ See L<EXPORT_OK>
 
 Exports all available exportable functions.
 
-=head1 FUNCTION TOC
+=head1 FUNCTION TABLE OF CONTENTS
 
 =head2 CORE
 
@@ -434,6 +477,10 @@ See L</ADC FUNCTIONS>.
 
 See L</SHIFT REGISTER FUNCTIONS>.
 
+=head2 SPI
+
+See L</SPI FUNCTIONS>
+
 =head2 BAROMETRIC SENSOR
 
 See L</BMP180 PRESSURE SENSOR FUNCTIONS>.
@@ -875,6 +922,9 @@ Mandatory: A string to display.
 
 =head1 SOFT PWM FUNCTIONS
 
+Note: The software PWM functionality is experimental, and from what I've
+tested, not very reliable, so I'd stay away from this at this time.
+
 Software Pulse Width Modulation is not the same as hardware PWM. It should not
 be used for critical things as it's frequency isn't 100% stable.
 
@@ -949,7 +999,9 @@ This section is broken down by type/model.
 
 =head2 ADS1115 MODEL
 
-=head3 ads1115Setup($pin_base, $addr)
+=head3 ads1115_setup($pin_base, $addr)
+
+Maps to `ads1115Setup(int pinBase, int addr)`.
 
 The ADS1115 is a four channel, 16-bit wide ADC.
 
@@ -1022,6 +1074,64 @@ Mandatory: Integer, the GPIO pin number connected to the register's C<SHCP> pin
 Mandatory: Integer, the GPIO pin number connected to the register's C<STCP> pin
 (12). Can be any GPIO pin capable of output.
 
+=head1 SPI FUNCTIONS
+
+These functions allow you to set up and read/write to devices on the serial
+peripheral interface (SPI) bus.
+
+=head2 spi_setup
+
+Maps to C<int wiringPiSPISetup(int channel, int speed)>
+
+Configure the SPI bus for use to communicate with its connected devices.
+
+Parameters:
+
+    $channel
+
+Mandatory: Integer, the SPI channel the device is connected to. C<0> for channel
+C</dev/spidev0.0> and C<1> for channel C</dev/spidev0.1>.
+
+    $speed
+
+Optional: Integer, the speed for SPI communication. Defaults to 1000000 (1MHz).
+
+Note that it's wise to do some error checking when attempting to open the SPI
+bus. We return the return value of an C<ioctl()> call, so this does the trick:
+
+    if ((spi_setup(0, 1000000) < 0){
+        die "failed to open the SPI bus...\n";
+    }
+
+=head2 spi_data
+
+Maps to: C<int spiDataRW(int channel, AV* data, int len)>, which calls
+C<int wiringPiSPIDataRW(int channel, unsigned char* data, int len)>.
+
+Writes, and then reads a block of data over the SPI bus. The read following the
+write is read into the transmit buffer, so it'll be overwritten.
+
+Parameters:
+
+    $channel
+
+Mandatory: Integer, the SPI channel the device is connected to. C<0> for channel
+C</dev/spidev0.0> and C<1> for channel C</dev/spidev0.1>.
+
+    $data
+
+Mandatory: An array reference, with each element containing a single unsigned
+8-bit byte that you want to write to the device.
+
+    $len
+
+Mandatory: Integer, the number of bytes contained in the C<$data> parameter
+array reference that will be sent to the device. I could just count the number
+of elements, but this keeps things consistent, and ensures the user is fully
+aware of the data they are sending on the bus.
+
+Returns the write buffer (C<$data>) after it's been written with the read data.
+
 =head1 BMP180 PRESSURE SENSOR FUNCTIONS
 
 These functions configure and fetch data from the BMP180 pressure sensor.
